"""From the south pole met office you get wind speeds once every minute.  Take an average for 10 minutes and generate a transition matrix to give <10 minute average now> -> <average for the next 10 minutes>
"""

import glob
import os
import itertools
from weather import weatherdata
from scipy import sparse
import psyco
import random
import scipy
import math

def generateMatrix(data):

    # the data is supplied in a sorted list
    # re-arrange it so it's accessible by epoch
    dataDict = {}
    for pt in data:
        dataDict[int(pt[0])] = pt

    probDict = {}

    print "length of data: %d" % len(data)

    for pt in data:
        ptEpoch = pt[0]

        windSpeeds = []
        for i in range(10):
            newEpoch = int(ptEpoch+(i*60))

            if(newEpoch in dataDict):
                windSpeed = dataDict[newEpoch][2]
                windSpeeds.append(windSpeed)

        nextWindSpeeds = []
        for i in range(10,20):
            newEpoch = int(ptEpoch+(i*60))
            if(newEpoch in dataDict):
                windSpeed = dataDict[newEpoch][2]
                nextWindSpeeds.append(windSpeed)

        if(len(windSpeeds)==0 or len(nextWindSpeeds)==0):
            continue

        avgNow = sum(windSpeeds)/len(windSpeeds)
        avgNext = sum(nextWindSpeeds)/len(nextWindSpeeds)

        # scale the avg now and next to be in the right dimensions
        avgNow = int(avgNow*10.0)
        avgNext = int(avgNext*10.0)
            
        if(avgNow not in probDict):
            probDict[avgNow]={}

        if(avgNext not in probDict[avgNow]):
            probDict[avgNow][avgNext]=0

        probDict[avgNow][avgNext]+=1
        

    print "done looking at all the data"
    print "Found %d keys" % ( len(probDict.keys()))
    maxSpeed = max(probDict.keys())
    maxDim = int(maxSpeed)+1
    print "max speed: %f" % (maxSpeed/10.0)
    print "transition matrix dimensions: %d x %d" % ( maxDim, maxDim)
    
    a = sparse.lil_matrix((maxDim, maxDim))

    for i in range(maxDim):
        for j in range(maxDim):
            if(a[i,j]!=0):
                print "initial matrix value wrong!"

    
    # calculate the probability of the next wind speed given the current wind speed
    for i in probDict.keys():
        count = 0.
        for j in probDict[i].keys():
            count=count+probDict[i][j]
        if(count>0):
            for j in probDict[i].keys():
                results = probDict[i][j]/count
                a[i,j] = results

    return a



def loadData(dataDir, desiredMonth=None, numYears=None):
    years = []
    trainingFiles = glob.glob(os.path.join(dataDir,"SPole_OneMinSurf_CAIR*.txt"))
    for file in trainingFiles:
        year = weatherdata.getYear(file)
        if(year not in years):
            years.append(year)

    if(numYears!=None):
        numYears = min(numYears, len(years))
        years = years[0:numYears]

    weatherData = []
    for year in years:
        # cheat a bit and only ask for one month.  makes testing easier
        data = weatherdata.loadDataCAIR(year, dataDir, desiredMonth)
        print "data: len(data) = %d" % len(data)

        weatherData.extend(data)

    print "len weather data: %d" % len(weatherData)
    return weatherData
    

def testMatrix(transMatrix, testData, month):

    print "test matrix..."
    # the transition matrix is good for a 10 minute step
    # more than ten
    matrixDict={}
    for hourOffset in [0.5,1,2,4]:
        minutes = int( (hourOffset*60.0)/10.0)
        matrixDict[hourOffset] = transMatrix**minutes

    # the data is supplied in a sorted list
    # re-arrange it so it's accessible by epoch
    dataDict = {}
    print "about to go enter all data points in a dictionary"
    for pt in testData:
        dataDict[int(pt[0])] = pt

    resultsDict = {}

    rowLength = transMatrix.shape[1]
    print "Row Length: %d" % rowLength

    totalNumPts = 0
    totalPtsIdx = 0
    for pt in testData:
        if(totalNumPts>totalPtsIdx*500.):
            print "total num points: %d" % totalNumPts
            totalPtsIdx=totalPtsIdx+1
        totalNumPts=totalNumPts+1
        ptEpoch = pt[0]

        windSpeeds = []
        for i in range(10):
            newEpoch = int(ptEpoch+(i*60))

            if(newEpoch in dataDict):
                windSpeed = dataDict[newEpoch][2]
                windSpeeds.append(windSpeed)
        
        avgNow = sum(windSpeeds)/len(windSpeeds)
        avgNowIdx = int(avgNow*10.0)

        for hourOffset in [0.5,1,2,4]:
            nextWindSpeeds = []
            for i in range(10,20):
                newEpoch = int(ptEpoch+(i*60)+(hourOffset*60*60.))
                if(newEpoch in dataDict):
                    windSpeed = dataDict[newEpoch][2]
                    nextWindSpeeds.append(windSpeed)

            if(len(nextWindSpeeds)==0):
                # no data..
                continue
            avgNext = sum(nextWindSpeeds)/len(nextWindSpeeds)

            # scale the avg now and next to be in the right dimensions
            avgNextIdx = int(avgNext*10.0)
        
            stepMatrix = matrixDict[hourOffset]
            # get the predicted wind speed
            pick = random.random()
            total = 0.0
            predictedIdx = 0
            for idx in range(rowLength):
                total = total + stepMatrix[avgNowIdx,idx]
                if(total>pick):
                    # found the predicted wind speed
                    predictedIdx = idx
                    break
                

            predicted = predictedIdx / 10.0

            diff = predicted - avgNext
            #print "diff: %f, predicted: %f, next: %f" % ( diff, predicted, avgNext)

            if(not hourOffset in resultsDict):
                resultsDict[hourOffset] = [ diff ]
            else:
                resultsDict[hourOffset].append(diff)

    print "Done calculating results..."
    fd = open('gen-avg-output-%d.dat' % month, 'w')
    for offset in resultsDict.keys():
        count = len(resultsDict[offset])
        mean = scipy.mean(resultsDict[offset])
        stddev = scipy.std(resultsDict[offset])

        fd.write('%f %d %f %f\n' % ( offset, count, mean, stddev))
        print "%f %d %f %f\n" % ( offset, count, mean, stddev)
    fd.close()
                
if __name__=="__main__":
    psyco.full()
    data = loadData("./data/training")
    matrix = generateMatrix(data)

    # test a row of the transition matrix to make sure it adds up to 1.0
    total = 0
    for i in range(matrix.shape[1]):
        total = total + matrix[0,i]

    print "total should be one"
    print "total %f" % total

    # load the test data
    for month in range(4,9):
        print "month: %d" % month
        testData = loadData('./data/test', desiredMonth=month, numYears=1)
        testMatrix(matrix, testData, month)


    




