"""Generate a markov transition matrix.
Basically given all of the wind data in the training directory I want to come up with a matrix that
tells you if the current wind speed is X, here are the probabilities for what the next wind speed will be.
"""

import glob
import os
import itertools
from weather import weatherdata
from scipy import sparse
import psyco
import random
import scipy
import math

def generateMatrix(data):

    probDict = {}

    i1 = iter(data)
    i2 = iter(data)
    i2.next()
    pairs = itertools.izip(i1,i2)


    count = 0
    for pt1, pt2 in pairs:
        count +=1
        try:
            if( (pt2[0]-pt1[0])>90.):
                # skip if a large gap
                continue
        
            speed1 = pt1[2]
            speed2 = pt2[2]
            if(speed1 not in probDict):
                probDict[speed1]={}

            if(speed2 not in probDict[speed1]):
                probDict[speed1][speed2]=0

            probDict[speed1][speed2]+=1
        
        except Exception,e:
            print "Exception: %s" % e
            print "count: %d" % count
            print pt2
            print pt1


    maxSpeed = max(probDict.keys())
    maxDim = int(maxSpeed*10.0)+1
    print "max speed: %f" % maxSpeed
    print "transition matrix dimensions: %d x %d" % ( maxDim, maxDim)

    a = sparse.lil_matrix((maxDim, maxDim))

    # calculate the probability of the next wind speed given the current wind speed
    for speed in probDict:
        count = 0.
        i = int(speed*10.0)
        for nextSpeed in probDict[speed]:
            count+=probDict[speed][nextSpeed]
        if(count>0):
            for nextSpeed in probDict[speed]:
                j = int(nextSpeed*10.0)
                a[i,j] = probDict[speed][nextSpeed]/count
    

    return a



def loadData(dataDir):
    years = []
    trainingFiles = glob.glob(os.path.join(dataDir,"SPole_OneMinSurf*.txt"))
    for file in trainingFiles:
        year = weatherdata.getYear(file)
        if(year not in years):
            years.append(year)

    weatherData = []
    for year in [years[0]]:
        data = weatherdata.loadDataCAIR(year, dataDir)
        print "data: len(data) = %d" % len(data)

        weatherData.extend(data)

    print "len weather data: %d" % len(weatherData)
    return weatherData
    

def testMatrix(transMatrix, testDir):
    testData = loadData(testDir)
    testDict = {}

    resultsDict = {}
    

    for pt in testData:
        testDict[pt[0]] = pt

    print "test data points: %d" % len(testDict)

    transMatrixDict={}
    for step in range(1,10):
        transMatrixDict[step] = transMatrix**step
        resultsDict[step] = []

    print "Running transition matrix tests"
    for epoch in testDict.keys():
        for testDuration in range(1,10):
            testEpoch = int(epoch + (testDuration/2.*60))
            if(testEpoch in testDict):
                oldWindSpeed = testDict[epoch][2]
                newWindSpeed = testDict[testEpoch][2]

                if(math.isnan(oldWindSpeed) or math.isnan(newWindSpeed)):
                    continue

                windKey = int(oldWindSpeed * 10)
                #print "Wind key: %d diff: %f" % (windKey, (testEpoch-epoch)/60.)
                
                maxWindKey = transMatrixDict[testDuration].shape[0]-1
                windKey = min(maxWindKey, windKey)

                transitionRow = transMatrixDict[testDuration][windKey,:]
                
                num = random.random()
                total = 0
                limit = transitionRow.shape[1]
                for idx in range(limit):
                    total = total + transitionRow[0,idx]
                    if(total>=num):
                        #print "predicted wind speed %f, real %f" % ( idx/10.0, newWindSpeed)
                        answer = (idx/10.0)-newWindSpeed
                        if(math.isnan(answer)):
                            # bad answer
                            break
                        resultsDict[testDuration].append(answer)
                        break

    print "Calculating results"
    fd = open('forecast-results.out','w')
    for key in resultsDict.keys():
        if key in resultsDict:
            for pt in resultsDict[key]:
                if(math.isnan(pt)):
                    print "key: %d, has a point that is nan" % key

            m = sum(resultsDict[key])/len(resultsDict[key])
            s = scipy.std(resultsDict[key])
            
            if(math.isnan(m)):
                print "mean nan:"
                print key
            if(math.isnan(s)):
                print "std nan:"
                print key
                
            fd.write('%d\t%f\t%f\n' % ( key, m, s))


    fd.close()
    
                    

    

if __name__=="__main__":
    psyco.full()
    data = loadData("./data/training")
    matrix = generateMatrix(data)
    z = matrix**30

    testMatrix(z, "./data/test")


