import glob
import os
import os.path
import time
import re

class weatherdata:
    
    @staticmethod
    def timeStrToEpoch(date, timeStr):
        try:
            strTime = time.strptime("%s %s" % ( date, timeStr), "%m/%d/%y %H:%M:%S")
        except ValueError:
            try:
                strTime = time.strptime("%s %s" % ( date, timeStr), "%m/%d/%Y %H:%M:%S")
            except ValueError, e:
                raise e

        epoch = time.mktime(strTime)

        return epoch

    @staticmethod
    def getYear(filename):

        match = re.match(".*(\d\d)(\d\d)\.txt", filename)
        
        month, year = match.groups()

        year = int(year)

        if(year<20):
            return year+2000
        else:
            return year+1900

    @staticmethod
    def getMonth(filename):

        match = re.match(".*(\d\d)(\d\d)\.txt", filename)
        
        month, year = match.groups()

        return int(month)
        

    @staticmethod
    def isWinterData(filename):
       
        match = re.match(".*(\d\d)(\d\d)\.txt", filename)
        
        month, year = match.groups()

        month = int(month)
        year = int(year)
        
        if(month in [ 1,2,3, 10,11,12 ]):
            # sun is up
            #print "Sun is up.. "
            return False
        else:
            #print "Sun is down.."
            return True

    @staticmethod
    def loadDataCAIR(year, dir, desiredMonth=None):

        data = []

        files = glob.glob(os.path.join(dir, '*.txt'))
        
        for file in files:
            fileMonth = weatherdata.getMonth(file)
            fileYear = weatherdata.getYear(file)

            if(fileYear != year):
                #print "not the right year, skipping: %s" % file
                continue

            if(not weatherdata.isWinterData(file)):
                #print "not winter data skipping processing file: %s" % file
                continue

            if(desiredMonth!=None):
                if(desiredMonth!=fileMonth):
                    # skipping a non-interesting month
                    continue


            oldEpoch = None
            fd = open(file,'r')
            for line in fd:
                splitData = line.strip().split()
                
                if(len(splitData)!=13):
                    if(len(splitData)>0 and splitData[0]=="Date"):
                        continue
                    
                    print "Line too short ( expecting 13 got %d ): %s" % ( len(splitData), line)
                    print "%s" % splitData[3]
                   #print splitData
                    continue
        
                try:
                    skipLine=False
                    for idx in [3,4,5,6,10,11,12]:
                        if(splitData[idx]=='M'):
                            # bad sensor value..  skip
                            skipLine=True
                            break
                    if(skipLine):
                        continue

                
                    date = splitData[0]
                    if(date=='UTCDate' or date=='(UTC)' or date=='Date'):
                        #print "date line: %s" % date
                        continue

                    # check the date to make sure the year and month agree with the file name
                    match = re.match("(\d\d)\/\d\d\/(\d\d)", date)
                    month, year = match.groups()
                    year = int(year)+2000.
                    month = int(month)
                    if(month!=fileMonth or year!=fileYear):
                        # mismatched file month and date string month
                        # OR
                        # mismatched file year and date string year
                        print "file name: %s, date: %s" % (file, date)
                        continue

                    timeStr = splitData[1]
                    
                    epoch = weatherdata.timeStrToEpoch(date,timeStr)
                    
                    if(oldEpoch==None):
                        oldEpoch=epoch
                    # the last epoch is the data line previous to this one
                    # it should be less than a minute ago
                    # let it be up to an our before you really complain
                    if( (epoch-oldEpoch)>(3600)):
                        print "Data file gap: %3.2f minutes" % ( (epoch-oldEpoch)/60.)
                        continue
                    oldEpoch = epoch

                    windDir = float(splitData[3])
                    windSpeed = float(splitData[4])
                    
                    temp1 = float(splitData[5])
                    temp2 = float(splitData[6])
                    
                    p1 = float(splitData[10])
                    p2 = float(splitData[11])
                    p3 = float(splitData[12])
                    
                except ValueError,e:
                    print "Bad line: %s" % line
                    print "Exception: " , e
                    continue

            
                dataPt = ( epoch, windDir, windSpeed, min(temp1, temp2), min(p1,p2,p3))
                data.append(dataPt)

                if(windSpeed<0):
                    print "BAD WINDSPEED!"
                    pass

            fd.close()
        
        # sort the data by time
        data.sort(cmp=lambda x,y: cmp(x[0],y[0]))
    
        print "data loaded: %d points" % len(data)
        return data


